% Perform Asset Pricing tests, 20110706. 
%
% Input monthly data for factors and portfolios from mat files
% Output asset pricing tests


%% Define inputs
clear; clc; close all;

% input file
infile = 'portfolios_FULL'; % 2013 bench results 

% test portfolios
assetsname = 'capmbank25'; 


% pick factor model
% 1 = CAPM, 2 = CAPM+BANK, 3 = FF3, 4 = FF3+Bank
% 5 = FF3+Mom, 6 = FF3+Mom+Bank, 7 = FF3+Liq, 8 = FF3+Liq+Bank
% 9 = FF3+Mom+Liq, 10 = (same) + bank
% 20 = FF3 + dDDnonfin (Vassalou and Xing(2004)), 21 = (same) + bank
% 22 = FF3 + dVIX (Ang et al 2006), 23 = (same) + bank
% 24 = FF3 + Credit + Term (Chen Roll Ross 1986), 25 = (same) + bank
% 29 = FF3+VIX+Spreads, 30 = (same) + bank
pickmodel = 2;


% trim sample of asset returns
% (affects asset pricing tests, but not descriptive stats on factors)
% (YYYYMM, set equal to +- inf for no trimming)
switch 1
    case 1 % full sample
%         datebeg = -inf;  % for first date available
        datebeg = 196912; % first date with Bank betas portfolios
        % Use this same begdate for all cases to make them comparable
        dateend = inf; % no trimming
    case 2 % remove 2007 crisis
        datebeg = 196912;
        dateend = 200612; % before crisis
    case 3 % first 30 years
        datebeg = 196912;
        dateend = 199911;
    case 4 % last 30 years
        datebeg = 198301;
        dateend = inf;
end

% other options for fmt function
aptopt.shanken = 1;
aptopt.intercept = 0;
aptopt.plot = 0;
% window (in months) to compute time-varying betas in Fama-MacBeth
aptopt.windowbetas = 12*5; % 0 does FM with constant betas



%% Load data and subset for sample period

% retrieve factors and assets
load(infile);

% define test assets
assets = port.(assetsname);
[T,N] = size(assets);
assets = assets - repmat(factors.rf,1,N);  % excess returns

% define mimicking portfolios
aptopt.remp = assets;  

% trim sample
% (only need to trim assets because FactorModelTests() clears all NaN rows)
removerows = (dates<datebeg | dates>dateend);
assets(removerows,:) = nan;





%% Select factors

% rescale bank factor
factors.bank = factors.bank * 0.01;

% define model
switch pickmodel
    case 1
        aptopt.model = 'CAPM';
        aptopt.fnames = char('RMRF');
        pickf = [factors.rmrf];
    case 2
        aptopt.model = 'CAPM+Bank';
        aptopt.fnames = char('RMRF','BANK');
        pickf = [factors.rmrf, factors.bank];
        aptopt.nontradedf = 2;
        
    case 3
        aptopt.model = 'FF3';
        aptopt.fnames = char('RMRF','SMB','HML');
        pickf = [factors.rmrf, factors.smb, factors.hml];
    case 4
        aptopt.model = 'FF3+Bank';
        aptopt.fnames = char('RMRF','BANK','SMB','HML');
        pickf = [factors.rmrf, factors.bank, factors.smb, factors.hml];
        aptopt.nontradedf = 2;
        
    case 5
        aptopt.model = 'Momentum';
        aptopt.fnames = char('RMRF','SMB','HML','UMD');
        pickf = [factors.rmrf, factors.smb, factors.hml, factors.umd];
    case 6
        aptopt.model = 'Momentum+Bank';
        aptopt.fnames = char('RMRF','BANK','SMB','HML','UMD');
        pickf = [factors.rmrf, factors.bank,...
            factors.smb, factors.hml, factors.umd];
        aptopt.nontradedf = 2;
        
    case 7
        aptopt.model = 'Liquidity';
        aptopt.fnames = char('RMRF','SMB','HML','LIQ');
        pickf = [factors.rmrf, factors.smb, factors.hml, factors.liq];
    case 8
        aptopt.model = 'Liquidity+Bank';
        aptopt.fnames = char('RMRF','BANK','SMB','HML','LIQ');
        pickf = [factors.rmrf, factors.bank,...
            factors.smb, factors.hml, factors.liq];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
        
    case 9
        aptopt.model = 'FF3+UMD+LIQ';
        aptopt.fnames = char('RMRF','SMB','HML','UMD','LIQ');
        pickf = [factors.rmrf, factors.smb, factors.hml,...
            factors.umd, factors.liq];
    case 10
        aptopt.model = 'FF3+UMD+LIQ+Bank';
        aptopt.fnames = char('RMRF','BANK','SMB','HML','UMD','LIQ');
        pickf = [factors.rmrf, factors.bank,...
            factors.smb, factors.hml, factors.umd, factors.liq];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
        
    case 20 % Vassalou and Xing (2004). They also try market + dDDnonfin
        aptopt.model = 'DDnonfin';
        aptopt.fnames = char('RMRF','SMB','HML','dDDnf');
        pickf = [factors.rmrf,factors.smb,factors.hml,factors.DDnonfin];
        aptopt.nontradedf = 4;
    case 21
        aptopt.model = 'Bank + DDnonfin';
        aptopt.fnames = char('RMRF','BANK','SMB','HML','dDDnf');
        pickf = [factors.rmrf, factors.bank,...
            factors.smb,factors.hml, factors.DDnonfin];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
        
    case 22 % Ang et al (2006). They also try FF3+UMD+LIQ+dVIX
        aptopt.model = 'dVIX';
        aptopt.fnames = char('RMRF','SMB','HML','dVIX');
        pickf = [factors.rmrf,factors.smb,factors.hml,factors.vix];
        aptopt.nontradedf = 4;
    case 23
        aptopt.model = 'Bank + dVIX';
        aptopt.fnames = char('RMRF','BANK','SMB','HML','dVIX');
        pickf = [factors.rmrf, factors.bank,...
            factors.smb,factors.hml, factors.vix];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
        
    case 24 % Chen, Roll, and Ross (1986)
        aptopt.model = 'Bond Spreads';
        aptopt.fnames = char('RMRF','SMB','HML','Credit','Term');
        pickf = [factors.rmrf,factors.smb,factors.hml,...
            factors.credit, factors.term];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
    case 25
        aptopt.model = 'Bank + Spreads';
        aptopt.fnames = char('RMRF','BANK','SMB','HML','Credit','Term');
        pickf = [factors.rmrf,factors.bank,factors.smb,factors.hml,...
            factors.credit, factors.term];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
        
    case 29 % VIX and Spreads
        aptopt.model = 'FF3+VIX+Spreads';
        aptopt.fnames = char('RMRF','SMB','HML','dVIX','Credit','Term');
        pickf = [factors.rmrf,factors.smb,factors.hml,...
            factors.vix, factors.credit, factors.term];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
    case 30
        aptopt.model = 'FF3+VIX+Spreads+Bank';
        aptopt.fnames = char('RMRF','BANK',...
            'SMB','HML','dVIX','Credit','Term');
        pickf = [factors.rmrf,factors.bank, factors.smb,factors.hml,...
            factors.vix, factors.credit, factors.term];
        warning('JPP:factors','More than 1 nontraded factor: ignore time-series tests');
end

[~,K] = size(pickf);


disp('Descriptive statistics on factors');
tempopt.normaltest = 0;
tempopt.unitroot = 0;
tempopt.plot = 0;
tempopt.corrlags = [1,2,3,6];
tempopt.vnames = aptopt.fnames;
DescStats(pickf,tempopt);

% test auto correlations
disp('Auto correlation for bank factor')
tempopt.displayLB = 0;
[r,~,q] = AutoCorrFn(factors.bank,6,tempopt);
tempout = [r, q(:,2:3)];
tempopt.cnames = char('Lag','Autocorr.','Ljung-Box','p-value');
mprint1(tempout,tempopt);



%% Asset pricing tests

% more inputs
aptopt.dates = dates;

% run tests
apt = FactorModelTests2014(assets,pickf,aptopt);




%% print nice summary output

fprintf(1,'\n\n\n *** Asset Pricing Tests Summary *** \n\n');
fprintf(1,'\nModel:%s  Assets:%s  Start:%d  End:%d  N.Obs:%d\n',...
    aptopt.model,assetsname,apt.dates(1),apt.dates(end),apt.T);
outinfo.rnames = char(...,
    ['Fama-MacBeth(w=',num2str(aptopt.windowbetas),')'],...
    'Coefficient(%)',...
    't-value',...
    'OLS',...
    'Coefficient(%)',...
    't-value',...
    'GLS',...
    'Coefficient(%)',...
    't-value',...
    'GMM(b_1st)',...
    'Coefficient',...
    't-value',...
    'GMM(b_iter)',...
    'Coefficient',...
    't-value');

if aptopt.intercept
    outinfo.cnames = char('Intercept',aptopt.fnames,'\chi^2','p-value');
    tempcols = K+3;
    outinfo.swidth = 140;
    
    outtab = [ nan(1,tempcols) ;
        apt.fm.lambdas'*100, apt.fm.chi2, apt.fm.chi2pval;
        apt.fm.lambdas_tstat', nan(1,2);
        nan(1,tempcols) ;
        apt.csols.lambdas'*100, apt.csols.chi2, apt.csols.chi2pval;
        apt.csols.lambdas_tstat', nan(1,2);
        nan(1,tempcols) ;
        apt.csgls.lambdas'*100, apt.csgls.chi2, apt.csgls.chi2pval;
        apt.csgls.lambdas_tstat', nan(1,2);
        nan(1,tempcols) ;
        nan, apt.gmm1.b', apt.gmm1.J, apt.gmm1.p;
        nan, apt.gmm1.t', nan(1,2);
        nan(1,tempcols) ;
        nan, apt.gmmi.b', apt.gmmi.J, apt.gmmi.p;
        nan, apt.gmmi.t', nan(1,2)];
else
    outinfo.cnames = char(aptopt.fnames,'\chi^2','p-value');
    tempcols = K+2;
    
    outtab = [ nan(1,tempcols) ;
        apt.fm.lambdas'*100, apt.fm.chi2, apt.fm.chi2pval;
        apt.fm.lambdas_tstat', nan(1,2);
        nan(1,tempcols) ;
        apt.csols.lambdas'*100, apt.csols.chi2, apt.csols.chi2pval;
        apt.csols.lambdas_tstat', nan(1,2);
        nan(1,tempcols) ;
        apt.csgls.lambdas'*100, apt.csgls.chi2, apt.csgls.chi2pval;
        apt.csgls.lambdas_tstat', nan(1,2);
        nan(1,tempcols) ;
        apt.gmm1.b', apt.gmm1.J, apt.gmm1.p;
        apt.gmm1.t', nan(1,2);
        nan(1,tempcols) ;
        apt.gmmi.b', apt.gmmi.J, apt.gmmi.p;
        apt.gmmi.t', nan(1,2)];
end

mprint1(outtab,outinfo);


% print nice table for many factors
if 0 && ~aptopt.intercept
    disp('GLS lambda (1st col) and Iterated GMM b (2nd col)')
    clear outinfo;
    temp = [cellstr(aptopt.fnames)'; repmat({'t-stat'},1,K)];
    temp = temp(:);
    outinfo.rnames = char(temp{:},'$\chi^2$','p-value');
    temp = [apt.csgls.lambdas'*100; apt.csgls.lambdas_tstat'];
    outtab = temp(:);
    temp = [apt.gmmi.b'; apt.gmmi.t'];
    outtab = [outtab, temp(:)];
    outtab = [outtab; ...
        apt.csgls.chi2, apt.gmmi.J;...
        apt.csgls.chi2pval, apt.gmmi.p];
    outinfo.fmt = '%4.2f';
    mprint1(outtab,outinfo);
end

fprintf(1,'\n\n\n\n');






