function fmt = FactorModelTests(re,f,options)
% Does asset pricing tests of linear factor models,
% following Cochrane's (2005) Asset Pricing book.
% The tests are time series, cross section, Fama-MacBeth, and
% GMM/SDF.
%
% INPUT:
%   re:         (T,N) excess returns on test portfolios
%   f:          (T,K) factors (do not include constant in input).
%                   If some factor is not traded, must indicate its
%                   column number in options.nontradedf.
%   options:    structure, fields [default], all optional:
%                   .windowbetas [0], #obs to estimate time-varying betas
%                       for Fama-MacBeth, default is constant beta
%                   .model, string with model's name
%                   .fnames, vector with factor names (don't use blanks)
%                   .nontradedf [0], column of f with nontraded factor.
%                       This factor will be replaced by the return on
%                       mimicking portfolio formed with re portfolios. Can
%                       only handle 1 nontraded factor.
%                       Default [0] means that all factors are traded
%                       excess returns.
%                   .remp [re], (T,anything), matrix with *excess* returns
%                       on portfolios used to create factor mimicking
%                       portfolio.
%                   .intercept [0], set =1 to include intercept in
%                       Fama-MacBeth and cross sectional OLS and GLS.
%                       1 is more robust; 0 is more efficient.
%                   .shanken [1], does Shanken (1992) corrections in CS
%                       tests. (Does not matter much for monthly data.)
%                   .plot [222], plot predicted returns in figure number
%                       options.plot. Set to 0 for no plots.
%                   .dates [1,2,...,T], column with dates for re and f.
%
% OUPUT:
%   (most output is on screen, but also have the following)
%   fmt:        structure, with fields (THIS LIST IS INCOMPLETE):
%                   .T, number of observations (eg, months), after any
%                   sample trimming
%                   .dates, (T,1) dates remaining after any sample trimming
%                   .betas, (N,K) constant betas from full sample
%                       time-series regression of each re on original 
%                       (maybe nontraded) factors
%                   .SIGMAeps, (N,N) cov of errors of beta regs
%                   .y, (N,1) average re, after any sample trimming
%                   .ts, structure with time series results
%                       ts.rfmp, (Traw,1) ret factor mimicking portfolio
%                       ts.betas, (N,K) full sample betas with traded factors
%                   .csols, structure with Cross Sectional OLS results
%                   .csgls, structure with Cross Sectional GLS results
%                   .fm, structure with Fama-MabBeth results
%                   .gmm1, structure with GMM 1st stage results
%                   .gmmi, structure with GMM ITERATED results
%
% REQUIRES:
%   Mike Cliff's GMM toolbox
%   OLS (and maybe other fns) from spatialeconometrics.com
%
% Joao Pereira, pereirajpp@gmail.com
% Created:  2011.Jul.28
% Updated:  2011.Aug.18, re can have some NaNs, then full rows are removed.
%           2011.Sep.07, plot and output average and predicted returns
%           2013.Aug.05, Shanken as default option; reorganized code in
%           subfunctions; intercept in cs; time-series tests.
%           2013.Sep.04, mimicking portfolios for nontraded factor; tstats
%           on alphas from TS regressions.
%           2014.Jan.07, output returns for factor mimicking portfolio.



% ----------------------------------------------------------------
%% check inputs and compute common aux vars
% ----------------------------------------------------------------

% options
if nargin < 3; options.null = 1; end
if ~isfield(options,'windowbetas'); options.windowbetas = 0; end
if ~isfield(options,'model'); options.model = 'unnamed'; end
if ~isfield(options,'nontradedf'); options.nontradedf = 0; end
if ~isfield(options,'intercept'); options.intercept = 0; end
if ~isfield(options,'shanken'); options.shanken = 1; end
if ~isfield(options,'plot'); options.plot = 222; end
if ~isfield(options,'remp'); options.remp = re; end



fprintf(1,'\n\n===================================================\n\n');
fprintf(1,' Function: FactorModelTests \n');
fprintf(1,' Linear factor model being tested: %s \n\n',options.model);
fprintf(1,'===================================================\n\n');

% sizes
[T,N] = size(re);
Traw = T; % keep copy
[test,K] = size(f);
if T~=test; error('Ret and Factors have different lengths'); end;
[test,~] = size(options.remp);
if T~=test; error('Factor mimicking portfolios have length ~= T'); end;
if length(options.nontradedf)>1; 
    error('Can only handle 1 nontraded factor')
end
    
    
% create artficial dates if not supplied
if ~isfield(options,'dates')
    fmt.dates = (1:T)';
else
    fmt.dates = options.dates;
end
datesraw = fmt.dates; % keep copy

% remove NaN in re or remp or f 
remove = any(isnan([f,re,options.remp]),2); % missing returns
temp = sum(remove);
if temp > 0
    warning('JPP:NaN',...
        'Removed %d rows (of %d) of data due to missing assets or factors',...
        temp,T)
    re = re(~remove,:);
    options.remp = options.remp(~remove,:);
    f = f(~remove,:);
    fmt.dates = fmt.dates(~remove,:);
    [T,K] = size(f); % update sizes
end
fmt.T = T; % store values

% auxiliar vars
fmt.y = (mean(re))';  % (N,1) average returns

% name factors
if ~isfield(options,'fnames')
    options.fnames = [ repmat('factor_',K,1), num2str((1:K)') ];
end
% factor names for CS and FM
if options.intercept
    fnamesintercept = char('Intercept',options.fnames); % with intercept
else
    fnamesintercept = options.fnames; % without intercept
end



% ----------------------------------------------------------------
%% Time-series regressions (Cochrane, sec 12.1)
% ----------------------------------------------------------------
fprintf(1,'\n----- Time-series regressions for model %s ----- \n',...
    options.model);

% ts calculations with aux fn (use raw factors regardless of traded or not)
[ts,temps] = ts_auxfn(re,f);

% store output required for CS tests below
fmt.betas = ts.betas; % (N,K) constant betas with raw factors
fmt.betaststat = ts.betaststat; % (N,K) (auxiliar output)
fmt.SIGMAeps = temps; % (N,N) cov of errors from beta regs

% create mimicking porffolio for nontraded factor
if options.nontradedf > 0
    % output betas with raw factors (together with average returns)
    fprintf(1,'\n- Avg Return and Betas (with raw factors) for each asset -\n');
    info.rnames = char('Asset', num2str((1:N)') );
    info.cnames = char('AvgRet(%)',options.fnames);
    fprintf(1,'i) Point estimates:\n');
    temp = [fmt.y*100, ts.betas];
    mprint1(temp, info);
    fprintf(1,'ii) T-statistics:\n');
    temp = [nan(N,1), ts.betaststat];
    mprint1(temp, info);
    fprintf(1,'Number of observations: T = %d \n',T);
    
    % prefix mimicking portfolio with R_
    tempfnames = cellstr(options.fnames);
    tempfnames{options.nontradedf} = ['R_',tempfnames{options.nontradedf}];
    tempfnames = char(tempfnames);
    
    % compute mimicking portfolio
    fprintf(1,'\n- Computing mimicking portfolio %s for %s -\n',...
        tempfnames(options.nontradedf,:),options.fnames(options.nontradedf,:));
    % TS regression: nontradedfactor_t = a + re_t * b + e_t
    tempres = ols(f(:,options.nontradedf), [ones(T,1), options.remp]);
    fprintf(1,'Results of TS reg: nontradedfactor_t = a + re_t * b + e_t:');
    prt(tempres);
    % portfolio weights
    switch 2
        % Unconstrained(1) seems more correct option for excess returns,
        % but rescaling(2) guarantees that average return of mimicking port
        % and point estimate of alpha wrt other factors does not depend 
        % on the particular scale of the nontraded factor. T-stat of alpha
        % is the same in both cases.
        case 1
            % unconstrained weights 
            wfmp = tempres.beta(2:end);
        case 2
            % rescaled weights to sum up to 1
            wfmp = tempres.beta(2:end) / sum(tempres.beta(2:end)); 
            % Port ret is changed by factor of 1/sum(weights).
            % This changes level of FMP and betas for test assets,
            % though not alphas' t-stats nor GRS of test assets.
            fprintf(1,'Note: coef b rescaled s.t. sum(b)=1\n');
    end
    % portfolio return
    rfmp = options.remp * wfmp;  % (T,1)
    
    % replace nontraded factor with mimicking R_factor_t = re_t * b
    fstar = f;
    fstar(:,options.nontradedf) = rfmp;
    fprintf(1,'Nontraded factor replaced w/ R_factor_t = re_t * b\n');
    
    % output some stats
    fprintf(1,'Weights in factor mimicking portfolio:\n');
    fprintf(1,'%8.4f\n',wfmp);
    fprintf(1,'Sum(b) = %6.4f \n',sum(wfmp));
    fprintf(1,'E[R_factor] = %6.4f%% \n',mean(rfmp*100));
    fprintf(1,'Std[R_factor] = %6.4f%% \n',std(rfmp*100));
    temp = corrcoef(rfmp,f(:,options.nontradedf));
    fprintf(1,'Correlation[R_factor,nontradedfactor] = %6.4f\n',temp(1,2));
    % alpha wrt other factors
    temp = 1:K;
    temp(options.nontradedf) = [];
    tempres = ols(rfmp,[ones(T,1), f(:,temp)]);
    %     fprintf(1,'Regression of R_factor on other factors (intercept is alpha):\n');
    %     prt(tempres);
    fprintf(1,'Alpha of R_factor wrt other factors = %6.4f%%, t-stat = %6.4f\n',...
        tempres.beta(1)*100, tempres.tstat(1));
    
    % overwrite ts using f*
    [ts,~] = ts_auxfn(re,fstar);
    
    % store returns for original raw dates   
    ts.rfmp = nan(Traw,1);
    [~, ia, ib] = intersect(datesraw,fmt.dates);
    ts.rfmp(ia) = rfmp(ib); 
    % this puts 0 outside intersection! Replace with nan:
    temp = setdiff((1:Traw)',ia);
    ts.rfmp(temp) = nan;
else
    tempfnames = options.fnames;
    ts.rfmp = []; % just in case is used by calling script
end

% store output from ts
fmt.ts = ts;

% output alphas and betas (together with average returns)
fprintf(1,'\n- Avg Return, Alpha, and Betas for each asset (assetXfactor) - \n');
info.rnames = char('Asset', num2str((1:N)') );
fprintf(1,'i) Point estimates:\n');
info.cnames = char('AvgRet(%)','Alpha(%)',tempfnames);
temp = [fmt.y*100, fmt.ts.alphas*100, ts.betas];
% note that these betas are with mimicking portfolios if there are
% nontraded factors
mprint1(temp, info);
fprintf(1,'ii) T-statistics:\n');
info.cnames = char('AvgRet(%)','Alpha',tempfnames);
temp = [nan(N,1), ts.alphaststat, ts.betaststat];
mprint1(temp, info);
fprintf(1,'Number of observations: T = %d \n',T);

% output GRS test
fprintf(1,'\n- Gibbons, Ross, and Shanken (1989) test:\n');
fprintf(1,'GRS F_{%d,%d} = %6.4f \n',ts.grsdf(1),ts.grsdf(2),ts.grs);
fprintf(1,'p-value = %6.4f \n',ts.grspval);

% output High-Low portfolio
fprintf(1,'\n- Spread on extreme portfolios:\n');
fprintf(1,'Avg Return, Alpha, and Betas for Last-First portfolio\n');
info.cnames = char('AvgRet(%)','Alpha(%)',tempfnames);
info.rnames = char('Hi-Lo','Point','t-stat');
temp = [ts.hilo.Ere*100, ts.hilo.alpha*100, ts.hilo.betas';...
    ts.hilo.Eretstat, ts.hilo.alphatstat, ts.hilo.betaststat'];
mprint1(temp, info);


% :::::::::::::::   Auxiliar subfunctions   :::::::::::::::

    function [ts,SIGMAeps] = ts_auxfn(re,f)
        % Time-series tests
        % OUT:
        %       SIGMAeps, (N,N) % cov of errors of beta regs
        %           (Sigma in Coch 12.6)
        %       ts, structure with results, including:
        %           ts.betas, (N,K) full sample constant betas
        %           ts.alphas, (N,1)
        
        % auxiliar vars
        [T,N] = size(re); %
        [~,K] = size(f); %
        
        % estimate constant full sample betas for each asset
        ts.betas = nan(N,K);
        ts.betaststat = nan(N,K);
        epsilons = nan(T,N); % errors of time series regressions
        ts.alphas = nan(N,1);
        ts.alphaststat = nan(N,1);
        for n = 1:N % cycle assets
            % single time series regression to estimate constant beta
            res = ols(re(:,n), [ones(T,1), f]);
            ts.alphas(n,1) = res.beta(1);
            ts.alphaststat(n,1) = res.tstat(1);
            ts.betas(n,:) = res.beta(2:end);
            ts.betaststat(n,:) = res.tstat(2:end);
            epsilons(:,n) = res.resid;
        end
        
        % High-Low portfolio (last-first portofolio)
        diff = re(:,end) - re(:,1);
        % excess return
        res = ols(diff,ones(T,1));
        ts.hilo.Ere = res.beta;
        ts.hilo.Eretstat = res.tstat;
        % alpha wrt all factors
        res = ols(diff,[ones(T,1), f]);
        ts.hilo.alpha = res.beta(1);
        ts.hilo.alphatstat = res.tstat(1);
        % also store betas
        ts.hilo.betas = res.beta(2:end);
        ts.hilo.betaststat = res.tstat(2:end);
        
        % cov of errors of beta regs (Sigma in Coch sec 12.1 & 12.2):
        SIGMAeps = cov(epsilons);   % N,N
        % this is the same as epsilons'*epsilons / (T-1)
        
        % Gibbons, Ross, and Shanken (1989), formula 12.6:
        SIGMAf = cov(f); % (K,K) Factors cov mat
        Ef = (mean(f))'; % (K,1) factors mean
        ts.grs = (T-N-K)/N * (1 + Ef'/SIGMAf*Ef)^-1 * ...
            ts.alphas'*pinv(SIGMAeps)*ts.alphas;
        ts.grsdf = [N, T-N-K];
        ts.grspval = 1 - cdf('F',ts.grs,ts.grsdf(1),ts.grsdf(2));
        
        % just for testing:
        if K == 1
            ts.F = (T-N-1)/N * (1 + (mean(f)/std(f))^2 )^-1 * ...
                ts.alphas' * SIGMAeps^-1 * ts.alphas;
            ts.Fpval = 1 - cdf('F',ts.F,N,T-N-1);
            ts.chi2 = T * (1 + (mean(f)/std(f))^2 )^-1 * ...
                ts.alphas' * SIGMAeps^-1 * ts.alphas;
            ts.chi2pval = 1 - chi2cdf(ts.chi2,N);
        end
        % result: F gives same value as 12.6, chi2 very close
    end



% ----------------------------------------------------------------
%% Cross-sectional regressions (Cochrane, Sec 12.2)
% ----------------------------------------------------------------
fprintf(1,'\n----- Cross-sectional regressions for model %s ----- \n',...
    options.model);

% OLS calculations with aux fn
csols = csols_auxfn(re,f,options,fmt.betas,fmt.SIGMAeps);
fmt.csols = csols; % store results

% output OLS results to screen
fprintf(1,'\n- OLS -\n');
fprintf(1,'(Errors in Beta estimation assumed iid, though correlated across assets)\n');
if options.shanken
    fprintf(1,'(Shanken correction for estimated Betas)\n');
end
% output nice table
outputlambdas(csols.lambdas',csols.lambdas_std',csols.lambdas_tstat',...
    csols.chi2,csols.chi2df,csols.chi2pval,nan,fnamesintercept);

% GLS calculations with aux fn
csgls = csgls_auxfn(re,f,options,fmt.betas,fmt.SIGMAeps);
fmt.csgls = csgls; % store results

% output GLS results to screen
fprintf(1,'- GLS (corrects residuals correlation in cs reg) -\n');
fprintf(1,'(Errors in Beta estimation assumed iid, though correlated across assets)\n');
if options.shanken
    fprintf(1,'(Shanken correction for estimated Betas)\n');
end
% output nice table
outputlambdas(csgls.lambdas',csgls.lambdas_std',csgls.lambdas_tstat',...
    csgls.chi2,csgls.chi2df,csgls.chi2pval,nan,fnamesintercept);

fprintf(1,'T = %d \n',fmt.T);

% :::::::::::::::   Auxiliar subfunctions   :::::::::::::::

    function csols = csols_auxfn(re,f,options,betas,SIGMAeps)
        % Cross-sectional OLS tests
        % IN:
        %       betas, (N,K) full sample constant betas
        %       SIGMAeps, (N,N) Cov mat of residuals of beta estimations
        
        % auxiliar vars
        y = (mean(re))';   % (N,1) average returns
        [T,N] = size(re); %
        [~,K] = size(f); %
        
        SIGMAf = cov(f); % (K,K) Factors cov mat
        
        if options.intercept
            betas = [ones(N,1), betas]; %(N,K+1)
            SIGMAf = [zeros(1,K+1); zeros(K,1) SIGMAf]; % (K+1,K+1)
        end
        
        % point estimates
        lambdas = (betas'*betas)\betas'*y; % K,1
        alphas = y - betas * lambdas; % N,1
        
        % auxiliar vars
        BB = betas'*betas; % (K,K)
        if options.intercept
            tempShanken = (1+lambdas'*pinv(SIGMAf)*lambdas);
            % for Shanken correction
            % with intercept, SIGMAf is singular due to extra zeros
        else
            tempShanken = (1+lambdas'/SIGMAf*lambdas);
            % for Shanken correction
        end
        
        if options.shanken
            % frml 12.19
            lambdascov = 1/T * ...
                (BB\betas'* SIGMAeps * betas/BB * tempShanken + SIGMAf);
        else
            % frml 12.12
            lambdascov = 1/T * (BB\betas'* SIGMAeps * betas/BB + SIGMAf);
        end
        
        temp = eye(N) - betas/BB*betas';
        if options.shanken
            % frml 12.20
            alphascov = 1/T * (temp * SIGMAeps * temp') * tempShanken;
        else
            % frml 12.13:
            alphascov = 1/T * temp * SIGMAeps * temp';
        end
        
        % Chi2 test - frml 12.14:
        csols.chi2 = alphas' * pinv(alphascov) * alphas;
        % cov matrix is singular
        if options.intercept
            csols.chi2df = N - (K+1); % Deg Freedom with K factors + intercept
        else
            csols.chi2df = N - K; % Deg Freedom with K factors
        end
        csols.chi2pval = 1 - chi2cdf(csols.chi2,csols.chi2df);
        
        % individual lambda's t-stats
        csols.lambdas_std = (diag(lambdascov).^.5);
        csols.lambdas_tstat = lambdas ./ csols.lambdas_std;
        
        % store output
        csols.lambdas = lambdas;
        csols.yhat = betas * lambdas; % N,1
    end

% :::::::::::::::   Auxiliar subfunctions   :::::::::::::::

    function csgls = csgls_auxfn(re,f,options,betas,SIGMAeps)
        % Cross-sectional GLS tests
        % IN:
        %       betas, (N,K) full sample constant betas
        %       SIGMAeps, (N,N) Cov mat of residuals of beta estimations
        
        % auxiliar vars
        y = (mean(re))';   % (N,1) average returns
        [T,N] = size(re); %
        [~,K] = size(f); %
        
        SIGMAf = cov(f); % (K,K) Factors cov mat
        
        if options.intercept
            betas = [ones(N,1), betas]; %(N,K+1)
            SIGMAf = [zeros(1,K+1); zeros(K,1) SIGMAf]; % (K+1,K+1)
        end
        
        % auxiliar vars
        BSB = betas'/SIGMAeps*betas; % (K,K)
        
        % Point estimates - frml 12.15
        lambdas =  (BSB)\ betas'/SIGMAeps * y; % K,1
        alphas = y - betas * lambdas; % N,1
        
        % auxiliar for Shanken corrections
        if options.intercept
            tempShanken = (1+lambdas'*pinv(SIGMAf)*lambdas);
            % with intercept, SIGMAf is singular due to extra zeros
        else
            tempShanken = (1+lambdas'/SIGMAf*lambdas);
        end
        
        if options.shanken
            % frml 12.19
            lambdascov = 1/T*( inv(BSB) * tempShanken + SIGMAf ); %#ok<MINV>
            % need inv() bc X\scalar does not work.
        else
            % frml 12.16
            lambdascov = 1/T * ( inv(BSB) + SIGMAf);
        end
        
        if options.shanken
            % frml 12.21:
            alphascov = 1/T * ( SIGMAeps - betas/(BSB)*betas' ) * tempShanken;
        else
            % frml 12.17:
            alphascov = 1/T * ( SIGMAeps - betas/(BSB)*betas' );
        end
        
        % Chi2 test - frml 12.14:
        csgls.chi2 = alphas' * pinv(alphascov) * alphas;
        if options.intercept
            csgls.chi2df = N - (K+1); % Deg Freedom with K factors + intercept
            % 20130807: I am not sure about df... Need to check this.
        else
            csgls.chi2df = N - K; % Deg Freedom with K factors
        end
        csgls.chi2pval = 1 - chi2cdf(csgls.chi2,csgls.chi2df);
        
        % individual lambda's t-stats
        csgls.lambdas_std = (diag(lambdascov).^.5);
        csgls.lambdas_tstat = lambdas ./ csgls.lambdas_std;
        
        % store output
        csgls.lambdas = lambdas;
        csgls.yhat = betas * lambdas; % N,1
    end



% ----------------------------------------------------------------
%% Fama-MacBeth tests (Cochrane, sec 12.3)
% ----------------------------------------------------------------
fprintf(1,'\n----- Fama-MacBeth tests for model %s ----- \n',options.model);
if options.windowbetas >0
    fprintf(1,'(time-varying betas estimated with %d obs)\n',options.windowbetas);
else
    fprintf(1,'(constant betas)\n');
end

% FM calculations with aux fn
fm = fm_auxfn(re,f,options,fmt.betas);
fmt.fm = fm; % store results

% output nice table
outputlambdas(fm.lambdas', fm.lambdas_std', fm.lambdas_tstat',...
    fm.chi2, fm.chi2df, fm.chi2pval, fm.rbar, fnamesintercept)
fprintf(1,'Number of cross sections = %d \n',fm.ncs);

% :::::::::::::::   Auxiliar subfunctions   :::::::::::::::

    function fm = fm_auxfn(re,f,options,betas)
        % Fama-MacBeth tests
        % IN:
        %       betas, (N,K) full sample constant betas
        
        % auxiliar vars
        [T,N] = size(re); %
        [~,K] = size(f); %
        
        % preallocate
        cs.betas = nan(N,K);
        cs = repmat(cs,T,1);
        
        % estimate FM betas
        if options.windowbetas == 0 % constant betas
            for t = 1:T % same betas for all t
                cs(t).betas = betas;
            end
        else     % time varying betas
            for n = 1:N % cycle assets
                for t = options.windowbetas : T
                    window = (t-options.windowbetas+1 : t)';
                    y = re(window,n); % excess return in this window
                    X = [ones(length(window),1),f(window,:)];
                    res = ols(y,X);
                    cs(t).betas(n,:) = res.beta(2:end);
                    % end
                end
            end
        end
        
        % cross-sectional regressions at each t
        if options.intercept
            lambdas = nan(T,K+1); % includes intercept.
        else
            lambdas = nan(T,K); % does not include intercept.
        end
        rbars = nan(T,1); % Note: do not report R2 if no intercept
        alphas = nan(T,N); % regression residuals
        firstcs = max(2,options.windowbetas+1);
        for t = firstcs : T
            y = re(t,:)';
            if options.intercept
                X = [ones(N,1), cs(t-1).betas]; % w/ intercept
            else
                X = cs(t-1).betas; % w/o intercept
            end
            res = ols(y,X);
            if all(~isnan(res.beta))
                lambdas(t,:) = res.beta';
                rbars(t,1) = res.rbar;
                alphas(t,:) = res.resid';
            end
        end
        
        % remove empty rows (months where did not estimate lambdas)
        remove = ( sum( isnan(lambdas) ,2 ) >0 );
        lambdas = lambdas(~remove,:);
        rbars = rbars(~remove);
        alphas = alphas(~remove,:);
        [fm.ncs, ~] = size(lambdas); % new number of cross sections (T)
        
        % lambdas
        fm.lambdas = mean(lambdas)';
        fm.lambdas_std = std(lambdas)' / (fm.ncs^0.5);
        fm.lambdas_tstat = fm.lambdas ./ fm.lambdas_std;
        
        % R2
        if options.intercept
            fm.rbar = nanmean(rbars);
        else
            fm.rbar = []; % do not report R2 if no intercept
        end
        
        % chi2 test - Cochrane(2005,p.247)
        [tempT, ~] = size(alphas);
        alphasmean = mean(alphas); % 1 x N
        alphasdev = alphas - repmat(alphasmean,tempT,1); % ncs x N
        alphascov = (alphasdev' * alphasdev) / tempT^2; % N x N
        fm.chi2 = alphasmean * pinv(alphascov) * alphasmean'; 
        if options.intercept
            fm.chi2df = N - (K+1); % Deg Freedom with K factors + 1 intercept
        else
            fm.chi2df = N - K; % Deg Freedom with K factors
        end
        fm.chi2pval = 1 - chi2cdf(fm.chi2,fm.chi2df);
    end



% ----------------------------------------------------------------
%% GMM  (Cochrane, sec 13.2)
% ----------------------------------------------------------------
fprintf(1,'\n----- GMM tests for model %s ----- \n',options.model);

% first stage coef from Cochrane sec 13.2
% temp = f;
temp = f - repmat(mean(f),T,1); % demeaned factors 
d = (re'*temp)/T;
b1 = (d'*d)\d'*mean(re)'; % use as initial guess in GMM below

% Cliff's GMM code
gmmauxhandle = @gmmaux;  % Function handle constructor
gmmopt.infoz.momt=gmmauxhandle;
gmmopt.W0 = 'I';
gmmopt.lags = 3;
gmmopt.plot = 0;
% 1st stage GMM
gmmopt.gmmit = 1;
gmmout1 = gmm(b1,gmmopt,re,f,ones(T,1));
% note that must feed 1 as intrument
fmt.gmm1 = gmmout1;
% Iterated GMM
gmmopt.gmmit = nan;
gmmouti = gmm(b1,gmmopt,re,f,ones(T,1));
fmt.gmmi = gmmouti;

% output
fprintf(1,'\n - SUMMARY OF GMM RESULTS - \n');
fprintf(1,'( b_j=0 means factor j not necessary given the other factors ) \n');
fprintf(1,'FIRST stage estimates:\n');
outputlambdas(gmmout1.b',gmmout1.se',gmmout1.t',...
    gmmout1.J,gmmout1.df,gmmout1.p,nan,options.fnames,'SDF_b');

fprintf(1,'ITERATED GMM estimates:\n');
outputlambdas(gmmouti.b',gmmouti.se',gmmouti.t',...
    gmmouti.J,gmmouti.df,gmmouti.p,nan,options.fnames,'SDF_b');


    function [m,e] = gmmaux(b,~,~,y,x,z)
        % Auxiliar moment conditions for Cliff's GMM code.
        % Nested here makes it available to GMM code outside through
        % handle.
        % IN:
        %   b,  (K,1) parameters for sdf
        %   y,  (T,N) excess returns
        %   x,  (T,K) factors
        %   z,  (T,nz) Intruments. Dont need them now, but code is general to
        %               accept them later.
        [T,N] = size(y);
        sdf = ones(T,1) - (x - repmat(mean(x),T,1))*b; % T,1
        % note that factors are demeaned
        e = y .* repmat(sdf,1,N); % T,N
        m = z'*e/T;  % nz,N
        m = m(:); % stack as column vec
    end





% ----------------------------------------------------------------
%% plot figures
% ----------------------------------------------------------------
if options.plot>0
    % define axis so that figures are comparable
    temp = [fmt.y, fmt.csols.yhat, fmt.csgls.yhat]*100;
    smallest = 0; %min(min(temp))*0.9, but this does not work when it is 0;
    largest = max(max(temp))*1.1;
    
    figure(options.plot)
    % OLS
    subplot(2,1,1);
    plot(fmt.y*100,fmt.csols.yhat*100,...
        'ks','MarkerSize',5,'MarkerFaceColor','b')
    title('OLS cross sectional regression')
    ylabel('Average re (%)'); xlabel('Predicted re (%)');
    ylim([smallest, largest]), xlim([smallest largest]);
    axis square;
    % GLS
    subplot(2,1,2);
    plot(fmt.y*100,fmt.csgls.yhat*100,...
        'ks','MarkerSize',5,'MarkerFaceColor','b')
    title('GLS cross sectional regression')
    ylabel('Average re (%)'); xlabel('Predicted re (%)');
    ylim([smallest, largest]), xlim([smallest largest]);
    axis square;
    
    % TODO: GMM
end
end




% ----------------------------------------------------------------
%% Other auxiliar sub functions
% ----------------------------------------------------------------

function outputlambdas(pt,stdev,tstat,chi2,chi2df,chi2pval,r2,fnames,...
    coefnames)
% Output nice tables
% IN:
%   coefnames,  optional, name of parameters, [Lambdas]
K = length(pt);
% labels
info.cnames =  char(fnames,['\chi^2_(',num2str(chi2df),')'],...
    'p-value','\bar{R^2}');
if nargin == 8
    info.rnames = char('Lambdas','Coefficient','std-error',...
        't-value');
else
    info.rnames = char(coefnames,'Coefficient','std-error',...
        't-value');
end
% lambdas
outtab = nan(3,K);
outtab(1,:) = pt;
outtab(2,:) = stdev;
outtab(3,:) = tstat;
% add chi 2
temp = [chi2,chi2pval ; nan(2,2) ];
outtab = [outtab,temp];
% temp = [chi2; nan(2,1); chi2pval];
% outtab = [outtab,temp];
% add r2
if ~isnan(r2)
    temp = [nanmean(r2);nan(2,1)];
    outtab = [outtab,temp];
end
% other options
info.swidth = 120;
% output
mprint1(outtab, info);
end

